import info.lotosummary.software.*;
import java.io.*;

/*
Rannu Java Version 1.08
Producing a random number table
Copyright 2017 Sogaya

JDK 1.8.0
JDK 1.3.1

*/

public class Rannu {
    static final double VERSION = 1.08;
    static final int ERROR_CODE = 1;
    static final int FROM = 400;
    static final int INTERVAL_FROM = 1;
    static final int INTERVAL_TO = 37;
    static final int LARGE_NUMBER = 2147483647;
    static final int MANAGE_TEXT_LINE_COUNT = 9;
    static final int MERSENNE_TWISTER = 1;
    static final int NOT_CONVERTED = -1;
    static final int XORSHIFT = 6;
    static final long KILOBYTES = 32L;
    static final long SEED = 123456L;
    static final String ERROR_TEXT = "error.txt";
    static final String FALSE = "0";
    static final String MANAGE_TEXT = "manage.txt";
    static final String RANNU_CSV = "rannu.csv";
    static final String RANNU_TEXT = "rannu.txt";

    static boolean csv, grouping, produce_rannu_text;
    static int from, interval_0, interval_1, lineCount, prng;
    static long kilo, seed;
    static String error, FULL_PATH_NAME, pathName;

    static String[] m = {
                      "", // m[0], FULL_PATH_NAME includes MANAGE_TEXT.
                     "0", // m[1], csv: RANNU_CSV
                  "C:\\", // m[2], pathName includes the directory name (C:\) and RANNU_TEXT.
                    "32", // m[3], kilo: RANNU_TEXT size (kilobytes, KB)
                     "1", // m[4], interval_0: Loto-7
                    "37", // m[5], interval_1: Loto-7
                     "0", // m[6], grouping: interval of 0-63
                   "400", // m[7], from: idling
                "123456", // m[8], seed: pseudorandom number generator (PRNG)
            "2147483647"  // m[9], lineCount: Produce lines regardless of kilobytes.
        };

    private static boolean readManageText() {
        m[0] = FULL_PATH_NAME;

        try {
            BufferedReader buffered = new BufferedReader( new FileReader(FULL_PATH_NAME) );
            for (int line = 1; line <= MANAGE_TEXT_LINE_COUNT; line += 1) {
                m[line] = buffered.readLine();
                if (m[line] == null) {
                    break;
                }

            }

            buffered.close();
            stringArrayToValues();
            return true;
        } catch(IOException e4) {
            error = "IOException e4: " + FULL_PATH_NAME;
            createErrorText(error);
            return false;
        }

    }

    private static boolean createErrorText(String messages) {
        try {
            BufferedWriter buffered = new BufferedWriter( new FileWriter(ERROR_TEXT) );
            buffered.write(messages); buffered.newLine();
            buffered.close();
            return true;
        } catch(IOException e6) {
            error = "IOException e6";
            return false;
        }

    }

    private static int stringToInteger(String s) {
        int result;

        try {
            result = Integer.parseInt(s);
            return result;
        } catch(NumberFormatException e1) {
            error = "NumberFormatException e1";
            return NOT_CONVERTED;
        }

    }

    private static String[] driveDirectoryBase(String s) {
        String SLASH = "/";
        String YEN = "\\";
        String[] UNDIVIDED = {"", "", "", "", ""};
        int firstTrace, trace, w;
        int s_length = s.length();
        String c;
        String SEPARATOR = YEN;
        String[] part = {"", "", "", "", ""};
        part[0] = s; // s.length > 0; in other words, part[0].length > UNDIVIDED[0].length

// s is divided into three parts: drive name, directory name, and base name.
        int k = 0;
        for( ; k < s_length; ++k) {
            c = s.substring(k, k + 1 );
            if( YEN.equals(c) ) {
                SEPARATOR = YEN;
                break;
            } else if( SLASH.equals(c) ) {
                SEPARATOR = SLASH;
                break;
            }

        }

        if(k == s_length) { // Neither YEN nor SLASH has been found.
            return UNDIVIDED;
        } else {
            w = k + 1;
            part[1] = s.substring(0, w); // C:\ for example
            trace = ++k;
            firstTrace = trace;
        }

        for( ; k < s_length; ++k) {
            c = s.substring(k, k + 1 );
            if( SEPARATOR.equals(c) ) {
                trace = ++k;
            }

        }

        part[2] = s.substring(firstTrace, trace);
        part[3] = s.substring(trace); // manage.txt for example
        part[4] = SEPARATOR;
        return part;
    }

    private static void stringArrayToValues() {
        if ( FALSE.equals(m[1]) ) {
            csv = false;
            pathName = m[2] + RANNU_TEXT;
        } else {
            csv = true;
            pathName = m[2] + RANNU_CSV;
        }

        kilo = stringToInteger(m[3]);
        interval_0 = stringToInteger(m[4]);
        interval_1 = stringToInteger(m[5]);

        grouping = ( FALSE.equals(m[6]) ) ? false : true;
        from = stringToInteger(m[7]); // idling
        seed = stringToInteger(m[8]);
        lineCount = stringToInteger(m[9]); // the first part of RANNU_TEXT
    }

    public Rannu() {
        produce_rannu_text = false;
        error = "No error";
        prng = MERSENNE_TWISTER; // MT
    }

    public static void main(String[] args) {
        boolean produce_rannu_text = true;
        boolean WITH_COMMENTS = true;
        boolean WITHOUT_COMMENTS = false;
        int count = args.length;
        int d;
        int prng = MERSENNE_TWISTER; // MT
        Numbered n;
        String argument;
        String BINGO_5 = "-b";
        String CSV = "-c";
        String EMPTY = "";
        String FROM_TERM = "-f"; // idling
        String GROUPING = "-g";
        String INTERVAL = "-i";
        String KILOBYTE_TERM = "-k";
        String LINES = "-L";
        String LOTO_6 = "-6";
        String LOTO_7 = "-7";
        String MINILOTO = "-m";
        String NEW_LINE = System.getProperty("line.separator");
        String PRNG_MT = "-pm";
        String PRNG_XORSHIFT = "-px";
        String REDIRECT = "-r";
        String REGARDLESS_OF_LINES = "-L-";
        String SEED_TERM = "-s";
        String version_comments = "-VC";
        String version_guide = "-V";
        int NEW_LINE_LENGTH = NEW_LINE.length();

        csv = false;
        from = FROM;
        grouping = false;
        interval_0 = INTERVAL_FROM;
        interval_1 = INTERVAL_TO;
        kilo = KILOBYTES;
        lineCount = LARGE_NUMBER;
        seed = SEED;

// C:\RannuJava\manage.txt for example
        String firstPart = System.getProperty("user.dir"); // current directory
        String[] name = {"", "", "", "", ""};
        name = driveDirectoryBase(firstPart);
        String root = name[1];
        String SEPARATOR = name[4];
        if(firstPart == root) { // C:\ for example
            FULL_PATH_NAME = root + MANAGE_TEXT;
        } else { // C:\RannuJava for example 
            FULL_PATH_NAME = firstPart + SEPARATOR + MANAGE_TEXT;
        }

        File file = new File(FULL_PATH_NAME);
        if( file.exists() ) { // MANAGE_TEXT
            readManageText();
        } else {
// Creating MANAGE_TEXT
            StringBuffer os = new StringBuffer();
            for(int line = 1; line <= MANAGE_TEXT_LINE_COUNT; line += 1 ) {
                os.append(m[line]);
                os.append(NEW_LINE);
            }

            String s = os.toString();
            try {
                BufferedWriter buffered = new BufferedWriter( new FileWriter(FULL_PATH_NAME) ); // MANAGE_TEXT
                buffered.write(s);
                buffered.close();
                System.exit(ERROR_CODE);
            } catch(IOException e5) {
                error = "IOException e5: " + FULL_PATH_NAME;
                createErrorText(error);
                System.exit(ERROR_CODE);
            }

        }

        for(int i = 0; i < count; ++i) {
            argument = args[i];

            if( BINGO_5.equals(argument) ) {
                interval_1 = 40;
            } else if( CSV.equals(argument) ) {
                csv = true;
            } else if( GROUPING.equals(argument) ) {
                grouping = true;
            } else if( LOTO_6.equals(argument) ) {
                interval_1 = 43;
            } else if( LOTO_7.equals(argument) ) {
                interval_1 = 37;
            } else if( MINILOTO.equals(argument) ) {
                interval_1 = 31;
            } else if( PRNG_MT.equals(argument) ) {
                prng = MERSENNE_TWISTER;
            } else if( PRNG_XORSHIFT.equals(argument) ) {
                prng = XORSHIFT;
            } else if( REDIRECT.equals(argument) ) {
                produce_rannu_text = false;
            } else if( REGARDLESS_OF_LINES.equals(argument) ) {
                lineCount = LARGE_NUMBER;
            } else if( version_comments.equals(argument) ) {
                guide(WITH_COMMENTS);
                System.exit(ERROR_CODE);
            } else if( version_guide.equals(argument) ) {
                guide(WITHOUT_COMMENTS);
                System.exit(ERROR_CODE);
            } else {
                d = argument.indexOf(FROM_TERM);
                if(d >= 0) {
                    n = new Numbered(argument);
                    from = n.number();
                    if( FROM_TERM.equals( n.parameter() ) ) {
                        continue;
                    }

                    from = stringToInteger(m[7]);
                }

                d = argument.indexOf(INTERVAL);
                if(d >= 0) {
                    n = new Numbered(argument);
                    interval_1 = n.number();
                    if( n.trimmed_sequence() ) {
                        interval_0 = n.subnumber();
                        if( INTERVAL.equals( n.parameter() ) ) {
                            continue;
                        }

                    }

                    interval_0 = stringToInteger(m[4]);
                    interval_1 = stringToInteger(m[5]);
                }

                d = argument.indexOf(KILOBYTE_TERM);
                if(d >= 0) {
                    n = new Numbered(argument);
                    kilo = n.number();
                    if( KILOBYTE_TERM.equals( n.parameter() ) ) {
                        continue;
                    }

                    kilo = stringToInteger(m[3]);
                }

                d = argument.indexOf(LINES);
                if(d >= 0) {
                    n = new Numbered(argument);
                    lineCount = n.number();
                    if( LINES.equals( n.parameter() ) ) {
                        continue;
                    }

                    lineCount = stringToInteger(m[9]);
                }

                d = argument.indexOf(SEED_TERM);
                if(d >= 0) {
                    n = new Numbered(argument);
                    seed = n.number();
                    if( SEED_TERM.equals( n.parameter() ) ) {
                        continue;
                    }

                    seed = stringToInteger(m[8]);
                }

            }

        }

// Producing a random number table
        Rannu rannu = new Rannu();
        rannu.run(kilo, seed, from, interval_0, interval_1, lineCount, prng, csv, grouping, produce_rannu_text);
        System.exit(0);
    }

    public static void run(long k, long s, int f, int i0, int i1, int l, int p, boolean c, boolean g, boolean pr) {
        csv = c;
        from = f;
        grouping = g;
        interval_0 = i0;
        interval_1 = i1;
        kilo = k;
        lineCount = l;
        prng = p;
        produce_rannu_text = pr;
        seed = s;

        RandomNumber r = new RandomNumber(kilo, seed, from, interval_0, interval_1, lineCount, prng, csv, grouping, produce_rannu_text);
        if ( r.createRannuText(pathName) ) {
            System.exit(0);
        } else {
            error = "r.createRannuText( " 
                + pathName 
                + " )";
            createErrorText(error);
            System.exit(ERROR_CODE);
        }

    }

    private static void guide(boolean comments) {
        if(comments) {
            System.err.println( "Rannu Java Version " + String.valueOf(VERSION) );
            System.err.println("");
            System.err.println("java Rannu          # Produce Rannu.txt ");
            System.err.println("java Rannu -6       # Loto-6 ");
            System.err.println("java Rannu -7       # Loto-7 ");
            System.err.println("java Rannu -b       # Bingo-5 ");
            System.err.println("java Rannu -c       # CSV ");
            System.err.println("java Rannu -f400    # idling ");
            System.err.println("java Rannu -g       # interval of 0 to 63 ");
            System.err.println("java Rannu -i1-37   # interval of 1 to 37 ");
            System.err.println("java Rannu -k32     # 32 kilobytes ");
            System.err.println("java Rannu -L20     # 20 lines ");
            System.err.println("java Rannu -L-      # Produce rannu.txt regardless of lines. ");
            System.err.println("java Rannu -m       # Miniloto ");
            System.err.println("java Rannu -pm      # MT ");
            System.err.println("java Rannu -px      # xorshift ");
            System.err.println("java Rannu -r       # standard output ");
            System.err.println("java Rannu -r > file_name # Redirect.");
            System.err.println("java Rannu -s123456       # seed ");
            System.err.println("java Rannu -s0      # Time is used as seed. ");
            System.err.println("java Rannu -V       # version ");
            System.err.println("java Rannu -VC      # version with comments");
        } else {
            System.err.println( "Rannu Java Version " + String.valueOf(VERSION) );
            System.err.println("");
            System.err.println("java Rannu          ");
            System.err.println("java Rannu -6       ");
            System.err.println("java Rannu -7       ");
            System.err.println("java Rannu -b       ");
            System.err.println("java Rannu -c       ");
            System.err.println("java Rannu -f400    ");
            System.err.println("java Rannu -g       ");
            System.err.println("java Rannu -i1-37   ");
            System.err.println("java Rannu -k32     ");
            System.err.println("java Rannu -L20     ");
            System.err.println("java Rannu -L-      ");
            System.err.println("java Rannu -m       ");
            System.err.println("java Rannu -pm      ");
            System.err.println("java Rannu -px      ");
            System.err.println("java Rannu -r       ");
            System.err.println("java Rannu -r > file_name ");
            System.err.println("java Rannu -s123456       ");
            System.err.println("java Rannu -s0      ");
            System.err.println("java Rannu -V       ");
            System.err.println("java Rannu -VC      ");
        }

    }

}
