package info.lotosummary.software;

import java.util.*;

public class RandomNumber {
    static final double N_2_31 = 2147483648.0; // 2^31 = 2147483648
    static final int B00000001 = 1;
    static final int B00000010 = 2;
    static final int B00000100 = 4;
    static final int B00001000 = 8;
    static final int B00010000 = 16;
    static final int B00100000 = 32;
    static final int BOOL_STREAM_SIZE = 192; // 32 * GROUP_SIZE
    static final int FIELD_COUNT = 20;
    static final int GROUP_SIZE = 6;
    static final int KEY_LENGTH = 4;
    static final int LARGE_NUMBER = 2147483647;
    static final int MASK = 0x00000001;
    static final int MERSENNE_TWISTER = 1;
    static final int NOT_CONVERTED = -1;
    static final int XORSHIFT = 6;
    static final long KB = 1024L;
    static final long KILOBYTES = 32L;
    static final long TEN_DIGITS = 1000000000L;
    static final String RANNU_CSV = "rannu.csv";
    static final String RANNU_TXT = "rannu.txt";
    static final String[] b = {"00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", "41", "42", "43", "44", "45", "46", "47", "48", "49", "50", "51", "52", "53", "54", "55", "56", "57", "58", "59", "60", "61", "62", "63", "64", "65", "66", "67", "68", "69", "70", "71", "72", "73", "74", "75", "76", "77", "78", "79", "80", "81", "82", "83", "84", "85", "86", "87", "88", "89", "90", "91", "92", "93", "94", "95", "96", "97", "98", "99"};

    boolean csv, grouping, produce_rannu_text;
    int from, interval_0, interval_1, lineCount, prng;
    long kilo, seed;
    PRNG prn;
    String error, rannuText, separator;

    private String[] driveDirectoryBase(String s) {
        String SLASH = "/";
        String YEN = "\\";
        String[] UNDIVIDED = {"", "", "", "", ""};
        int firstTrace, trace, w;
        int s_length = s.length();
        String c;
        String SEPARATOR = YEN;
        String[] part = {"", "", "", "", ""};
        part[0] = s; // s.length > 0; in other words, part[0].length > UNDIVIDED[0].length

// s is divided into three parts: drive name, directory name, and base name.
        int k = 0;
        for( ; k < s_length; ++k) {
            c = s.substring(k, k + 1 );
            if( YEN.equals(c) ) {
                SEPARATOR = YEN;
                break;
            } else if( SLASH.equals(c) ) {
                SEPARATOR = SLASH;
                break;
            }

        }

        if(k == s_length) { // Neither YEN nor SLASH has been found.
            return UNDIVIDED;
        } else {
            w = k + 1;
            part[1] = s.substring(0, w); // C:\ for example
            trace = ++k;
            firstTrace = trace;
        }

        for( ; k < s_length; ++k) {
            c = s.substring(k, k + 1 );
            if( SEPARATOR.equals(c) ) {
                trace = ++k;
            }

        }

        part[2] = s.substring(firstTrace, trace);
        part[3] = s.substring(trace); // manage.txt for example
        part[4] = SEPARATOR;
        return part;
    }

    public int random_integer() {
        return prn.nextRandom();
    }

    public void setPRNG() {

        switch (prng) {
        case MERSENNE_TWISTER:
            prn = new MT(seed);
            break;
        case XORSHIFT:
            prn = new Xorshift(seed);
            break;
        default:
            break;
        }

// idling
        for(int k = 0; k < from; ++k) { 
            random_integer();
        }

    }

    public boolean createRannuText(String pathName) { // rannuText (RANdom-NUmber table TEXT)
        double difference, range;
        int k, plus;
        long bytes, size;
        String item;

        String[] name = {"", "", "", "", ""};
        name = driveDirectoryBase(pathName);
        String directoryName = name[1] + name[2]; // C:\RannuJava\ for example

        if(csv) {
            separator = ",";
            rannuText = RANNU_CSV;
        } else {
            separator = " ";
            rannuText = RANNU_TXT;
        }

// Redirecting unless rannuText is opened.
        Feedable f;
        if(produce_rannu_text) {
            pathName = directoryName + rannuText;
            if(csv) {
                System.out.println(pathName);
            }

            if(lineCount == LARGE_NUMBER) {
                f = new FileOutputNewLine(pathName);
            } else {
                f = new FileOutputNewLineCount(pathName, lineCount);
            }

        } else {
            if(lineCount == LARGE_NUMBER) {
                f = new StandardOutputNewLine();
            } else {
                f = new StandardOutputNewLineCount(lineCount);
            }

        }

        f.setting(separator, FIELD_COUNT);

        if(seed == 0L) {
            Calendar now = Calendar.getInstance();
            int year = now.get(Calendar.YEAR);
            int month = now.get(Calendar.MONTH);
            int day = now.get(Calendar.DATE);
            int hour = now.get(Calendar.HOUR);
            int minute = now.get(Calendar.MINUTE);
            int second = now.get(Calendar.SECOND);
            int add = minute * 60 + hour * 60 * 60;
            int year_month_day_43 = (year + month + day) * 43;
            second += add;
            second += year_month_day_43;
            seed = (long)second;
        }

        setPRNG(); // from, seed

        if(interval_1 == 63) {
            if(interval_0 == 0) {
                grouping = true;
            }

        }

        if(grouping) {
// Buffering and grouping so that numbers can range from 0 to 63.
            int[] boolStream = new int[BOOL_STREAM_SIZE];
            int bits, u;
            int[] shift = {
                B00000001,
                B00000010,
                B00000100,
                B00001000,
                B00010000,
                B00100000
            };

            k = 0;
            bytes = kilo * KB;
            size = 0L;
            while(size <= bytes) {
// Buffering
                u = prn.nextRandom(); // -2147483648 to +2147483647
                for(int i = 0; i < 32; ++i) {
                    boolStream[k++] = MASK & u;
                    u >>= 1;
                }

                if(k < BOOL_STREAM_SIZE) {
                    continue;
                }

// Grouping
                for(k = 0; k < BOOL_STREAM_SIZE; ) {
                    bits = 0;
                    for(int i = 0; i < GROUP_SIZE; ++i) {
                        if(boolStream[k++] == 1) {
                            bits |= shift[i];
                        }

                    }

                    item = b[bits];
                    size = f.writing(item);
                }

                k = 0;
            }

        } else {
// Selecting an item and writing
            double g;

            bytes = kilo * KB;
            difference = (double)( interval_1 - interval_0 + 1 );
            range = difference/4294967296.0;
            size = 0L;
            while(size <= bytes) {
                g = (double)prn.nextRandom();
                g += N_2_31; // 0 to 4294967295
                plus = (int)Math.floor( range * g );
                k = interval_0 + plus;
                item = b[k];
                size = f.writing(item);
            }

        }

        f.ending(csv);
        return true;
    }

    public RandomNumber(long k, long s, int f, int i0, int i1, int l, int p, boolean c, boolean g, boolean pr) {
        csv = c;
        from = f;
        grouping = g;
        interval_0 = i0;
        interval_1 = i1;
        kilo = k;
        lineCount = l;
        prng = p;
        produce_rannu_text = pr;
        seed = s;
    }

}
